@@ -91,18 +91,19 @@ call the "cleanup" method on their esky.
 from __future__ import with_statement
 
 __ver_major__ = 0
-__ver_minor__ = 5
-__ver_patch__ = 3
+__ver_minor__ = 6
+__ver_patch__ = 0
 __ver_sub__ = ""
 __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,__ver_patch__,__ver_sub__)
 
 
-import sys
 import os
+import sys
 import shutil
 import errno
 import socket
 import time
+from functools import wraps
 
 try:
     import threading
@@ -115,9 +116,28 @@ if sys.platform != "win32":
 from esky.errors import *
 from esky.fstransact import FSTransaction
 from esky.finder import DefaultVersionFinder
+import esky.helper
+from esky.helper import EskyHelperApp
 from esky.util import split_app_version, join_app_version,\
                       is_version_dir, is_uninstalled_version_dir,\
-                      parse_version, get_best_version, appdir_from_executable
+                      parse_version, get_best_version, appdir_from_executable,\
+                      copy_ownership_info
+
+
+
+def use_helper_app(func):
+    """Method decorator to transparently use an esky's helper app, if present.
+
+    This decorator wraps an Esky method so that it is transparently proxied
+    to the helper process whenever the esky's "helper_app" attribute is set.
+    """
+    @wraps(func)
+    def method_using_helper_app(self,*args,**kwds):
+        if self.helper_app is not None:
+            return getattr(self.helper_app,func.func_name)(*args,**kwds)
+        return func(self,*args,**kwds)
+    return method_using_helper_app
+
 
 
 class Esky(object):
@@ -153,6 +173,8 @@ class Esky(object):
         self.reinitialize()
         self._lock_count = 0
         self.version_finder = version_finder
+        self.helper_app = None
+        self.HelperAppClass = EskyHelperApp
 
     def _get_version_finder(self):
         return self.__version_finder
@@ -169,6 +191,14 @@ class Esky(object):
         """Get the directory path in which self.version_finder can work."""
         return os.path.join(self.appdir,"updates")
 
+    def get_abspath(self,relpath):
+        """Get the absolute path of a file within the current version."""
+        if self.active_version:
+            v = join_app_version(self.name,self.active_version,self.platform)
+        else:
+            v = join_app_version(self.name,self.version,self.platform)
+        return os.path.abspath(oss.path.join(self.appdir,v,relpath))
+
     def reinitialize(self):
         """Reinitialize internal state by poking around in the app directory.
 
@@ -256,6 +286,20 @@ class Esky(object):
             os.unlink(os.path.join(lockdir,myid))
             os.rmdir(lockdir)
 
+    @use_helper_app
+    def has_root(self):
+        """Check whether the user currently has root/administrator access."""
+        return esky.helper.has_root()
+
+    def get_root(self):
+        """Attempt to gain root/administrator access by spawning helper app."""
+        if self.has_root():
+            return True
+        self.helper_app = self.HelperAppClass(self,as_root=True)
+        if not self.helper_app.has_root():
+            raise OSError(None,"could not escalate to root privileges")
+
+    @use_helper_app
     def cleanup(self):
         """Perform cleanup tasks in the app directory.
 
@@ -357,13 +401,32 @@ class Esky(object):
         version = self.find_update()
         if version is not None:
             assert parse_version(version) > parse_version(self.version)
-            self.fetch_version(version)
-            self.install_version(version)
+            #  Try to install the new version.  If it fails with
+            #  a permission error, escalate to root and try again.
             try:
-                self.uninstall_version(self.version)
-            except VersionLockedError:
-                pass
-            self.reinitialize()
+                self.fetch_version(version)
+                self.install_version(version)
+                try:
+                    self.uninstall_version(self.version)
+                except VersionLockedError:
+                    pass
+            except EnvironmentError, e:
+                if e.errno != errno.EACCES or self.has_root():
+                    raise
+                exc_type,exc_value,exc_traceback = sys.exc_info()
+                try:
+                    self.get_root()
+                except Exception, e:
+                    raise exc_type,exc_value,exc_traceback
+                else:
+                    self.fetch_version(version)
+                    self.install_version(version)
+                    try:
+                        self.uninstall_version(self.version)
+                    except VersionLockedError:
+                        pass
+            else:
+                self.reinitialize()
 
     def find_update(self):
         """Check for an available update to this app.
@@ -382,13 +445,21 @@ class Esky(object):
                 best_version = version
         return best_version
 
+    @use_helper_app
     def fetch_version(self,version):
         """Fetch the specified updated version of the app."""
         if self.version_finder is None:
             raise NoVersionFinderError
-        if not self.version_finder.has_version(self,version):
-            self.version_finder.fetch_version(self,version)
+        #  Get the new version using the VersionFinder
+        loc = self.version_finder.has_version(self,version)
+        if not loc:
+            loc = self.version_finder.fetch_version(self,version)
+        #  Adjust permissions to match the current version
+        vdir = join_app_version(self.name,self.version,self.platform)
+        copy_ownership_info(os.path.join(self.appdir,vdir),loc)
+        return loc
 
+    @use_helper_app
     def install_version(self,version):
         """Install the specified version of the app.
 
@@ -400,7 +471,8 @@ class Esky(object):
         target = join_app_version(self.name,version,self.platform)
         target = os.path.join(self.appdir,target)
         if not os.path.exists(target):
-            source = self.version_finder.fetch_version(self,version)
+            self.fetch_version(version)
+            source = self.version_finder.has_version(self,version)
         self.lock()
         try:
             if not os.path.exists(target):
@@ -427,6 +499,7 @@ class Esky(object):
         finally:
             self.unlock()
 
+    @use_helper_app
     def uninstall_version(self,version): 
         """Uninstall the specified version of the app."""
         target_name = join_app_version(self.name,version,self.platform)
@@ -455,15 +528,9 @@ class Esky(object):
                         to_keep.update(self._version_manifest(vname))
                     #  Remove files used only by the version being removed
                     to_rem = self._version_manifest(target_name) - to_keep
-                    for nm in os.listdir(self.appdir):
-                        #  Also remove temporarily-renamed ".old" files.
-                        while ".old." in nm or nm.endswith(".old"):
-                            if nm in to_rem:
-                                break
-                            idx = nm.find(".old")
-                            nm = nm[:idx] + nm[idx+4:]
-                        if nm in to_rem:
-                            fullnm = os.path.join(self.appdir,nm)
+                    for nm in to_rem:
+                        fullnm = os.path.join(self.appdir,nm)
+                        if os.path.exists(fullnm):
                             trn.remove(fullnm)
                 except Exception:
                     trn.abort()
@@ -514,3 +581,47 @@ class Esky(object):
         except IOError:
             return set()
 
+
+
+def _check_needsroot(func):
+    @wraps(func)
+    def do_check_needsroot(self,*args,**kwds):
+        if os.environ.get("ESKY_NEEDSROOT",""):
+            if not self.has_root():
+                raise OSError(errno.EACCES,"you need root")
+        return func(self,*args,**kwds)
+    return do_check_needsroot
+
+
+class _TestableEsky(Esky):
+    """Esky subclass that tries harder to be testable.
+
+    If the environment variable "ESKY_NEEDSROOT" is set, operations that
+    alter the filesystem will fail with EACCES when not executed as root.
+    """
+
+    @_check_needsroot
+    def lock(self,num_retries=0):
+        return super(_TestableEsky,self).lock(num_retries)
+
+    @_check_needsroot
+    def unlock(self):
+        return super(_TestableEsky,self).unlock()
+   
+    @_check_needsroot
+    def cleanup(self):
+        return super(_TestableEsky,self).cleanup()
+
+    @_check_needsroot
+    def fetch_version(self,version):
+        return super(_TestableEsky,self).fetch_version(version)
+
+    @_check_needsroot
+    def install_version(self,version):
+        return super(_TestableEsky,self).install_version(version)
+
+    @_check_needsroot
+    def uninstall_version(self,version):
+        super(_TestableEsky,self).uninstall_version(version)
+
+
