@@ -86,21 +86,27 @@ class Executable(str):
     def __new__(cls,script,**kwds):
         return str.__new__(cls,script)
 
-    def __init__(self,script,icon=None,gui_only=None,**kwds):
+    def __init__(self,script,name=None,icon=None,gui_only=None,
+                      include_in_bootstrap_env=True,**kwds):
         str.__init__(script)
         self.script = script
+        self.include_in_bootstrap_env = include_in_bootstrap_env
         self.icon = icon
+        self._name = name
         self._gui_only = gui_only
         self._kwds = kwds
 
     @property
     def name(self):
-        nm = os.path.basename(self.script)
-        if nm.endswith(".py"):
-            nm = nm[:-3]
-        elif nm.endswith(".pyw"):
-            nm = nm[:-4]
-        if sys.platform == "win32":
+        if self._name is not None:
+            nm = self._name
+        else:
+            nm = os.path.basename(self.script)
+            if nm.endswith(".py"):
+                nm = nm[:-3]
+            elif nm.endswith(".pyw"):
+                nm = nm[:-4]
+        if sys.platform == "win32" and not nm.endswith(".exe"):
             nm += ".exe"
         return nm
 
@@ -233,6 +239,9 @@ class bdist_esky(Command):
                     yield s
                 else:
                     yield Executable(s)
+            #  Include the esky helper app in all distributions
+            h = os.path.join(os.path.dirname(__file__),"../helper/__main__.py")
+            yield Executable(h,name="esky-update-helper",gui_only=True)
 
     def get_data_files(self):
         """Yield (source,destination) tuples for data files.
@@ -382,21 +391,24 @@ class bdist_esky(Command):
         #  Search for redist files in a Visual Studio install
         progfiles = os.path.expandvars("%PROGRAMFILES%")
         for dnm in os.listdir(progfiles):
-            if dnm.startswith("Microsoft Visual Studio"):
+            if dnm.lower().startswith("microsoft visual studio"):
                 dpath = os.path.join(progfiles,dnm,"VC","redist")
                 for (subdir,_,filenames) in os.walk(dpath):
                     for fnm in filenames:
-                        if name in fnm and fnm.endswith(".manifest"):
-                            yield os.path.join(subdir,fnm)
+                        if name.lower() in fnm.lower():
+                            if fnm.lower().endswith(".manifest"):
+                                yield os.path.join(subdir,fnm)
         #  Search for manifests installed in the WinSxS directory
         winsxs_m = os.path.expandvars("%WINDIR%\\WinSxS\\Manifests")
         for fnm in os.listdir(winsxs_m):
-            if name in fnm and fnm.endswith(".manifest"):
-                yield os.path.join(winsxs_m,fnm)
+            if name.lower() in fnm.lower():
+                if fnm.lower().endswith(".manifest"):
+                    yield os.path.join(winsxs_m,fnm)
         winsxs = os.path.expandvars("%WINDIR%\\WinSxS")
         for fnm in os.listdir(winsxs):
-            if name in fnm and fnm.endswith(".manifest"):
-                yield os.path.join(winsxs,fnm)
+            if name.lower() in fnm.lower():
+                if fnm.lower().endswith(".manifest"):
+                    yield os.path.join(winsxs,fnm)
 
     def copy_to_bootstrap_env(self,src,dst=None):
         """Copy the named file into the bootstrap environment.
