@@ -120,6 +120,8 @@ def freeze(dist):
     #  Copy the loader program for each script into the bootstrap env.
     dist.copy_to_bootstrap_env("Contents/MacOS/python")
     for exe in dist.get_executables():
+        if not exe.include_in_bootstrap_env:
+            continue
         exepath = dist.copy_to_bootstrap_env("Contents/MacOS/"+exe.name)
 
 
@@ -128,9 +130,22 @@ def _make_py2app_cmd(dist_dir,distribution,options,exe):
     for (nm,val) in options.iteritems():
         setattr(cmd,nm,val)
     cmd.dist_dir = dist_dir
-    cmd.app = [Target(script=exe,prescripts=[StringIO(_EXE_PRESCRIPT_CODE)])]
+    cmd.app = [Target(script=exe.script,dest_base=exe.name,
+                      prescripts=[StringIO(_EXE_PRESCRIPT_CODE)])]
     cmd.finalize_options()
     cmd.plist["CFBundleExecutable"] = exe.name
+    old_run = cmd.run
+    def new_run():
+        old_run()
+        #  We need to script file to have the same name as the exe, which
+        #  it won't if they have changed it explicitly.
+        resdir = os.path.join(dist_dir,distribution.get_name()+".app","Contents/Resources")
+        scriptf = os.path.join(resdir,exe.name+".py")
+        if not os.path.exists(scriptf):
+           old_scriptf = os.path.basename(exe.script)
+           old_scriptf = os.path.join(resdir,old_scriptf)
+           shutil.move(old_scriptf,scriptf)
+    cmd.run = new_run
     return cmd
 
 
