@@ -16,17 +16,21 @@ eskytester.yes_my_data_is_installed()
 
 #  Test that we're at the best possible version
 assert sys.frozen
-app = esky.Esky(sys.executable,"http://localhost:8000/dist/")
+app = esky._TestableEsky(sys.executable,"http://localhost:8000/dist/")
 assert app.name == "eskytester"
 assert app.active_version == app.version == "0.3"
 assert app.find_update() is None
 
+if os.environ.get("ESKY_NEEDSROOT",""):
+    print "GETTING ROOT"
+    app.get_root()
+    print "GOT ROOT"
+
 app.cleanup()
 assert os.path.isdir(os.path.join(app.appdir,"eskytester-0.3."+platform))
 assert os.path.isfile(eskytester.script_path(app,"script2"))
 assert os.path.isfile(eskytester.script_path(app,"script3"))
 
-
 #  Test that MSVCRT wasn't bundled with this version
 if sys.platform == "win32":
     for nm in os.listdir(os.path.dirname(sys.executable)):
@@ -40,6 +44,9 @@ if sys.platform == "win32":
 if sys.platform == "win32":
     assert hasattr(sys,"bootstrap_executable")
 
-open("tests-completed","w").close()
-print "TESTS COMPLETED"
+if sys.platform == "darwin":
+    open("../../../../tests-completed","w").close()
+else:
+    open("tests-completed","w").close()
+
 
